@@ -1,19 +1,21 @@
 //! Unification and canonicalization logic.
 
-use std::{fmt, mem, sync::Arc};
+use std::{fmt, iter, mem, sync::Arc};
 
 use chalk_ir::{
-    cast::Cast, fold::Fold, interner::HasInterner, zip::Zip, FloatTy, IntTy, TyVariableKind,
-    UniverseIndex,
+    cast::Cast, fold::Fold, interner::HasInterner, zip::Zip, FloatTy, IntTy, NoSolution,
+    TyVariableKind, UniverseIndex,
 };
 use chalk_solve::infer::ParameterEnaVariableExt;
 use ena::unify::UnifyKey;
+use hir_expand::name;
 
 use super::{InferOk, InferResult, InferenceContext, TypeError};
 use crate::{
-    db::HirDatabase, fold_tys, static_lifetime, AliasEq, AliasTy, BoundVar, Canonical, Const,
-    DebruijnIndex, GenericArg, Goal, Guidance, InEnvironment, InferenceVar, Interner, Lifetime,
-    ProjectionTy, Scalar, Solution, Substitution, TraitEnvironment, Ty, TyKind, VariableKind,
+    db::HirDatabase, fold_tys, static_lifetime, traits::FnTrait, AliasEq, AliasTy, BoundVar,
+    Canonical, Const, DebruijnIndex, GenericArg, Goal, Guidance, InEnvironment, InferenceVar,
+    Interner, Lifetime, ProjectionTy, ProjectionTyExt, Scalar, Solution, Substitution,
+    TraitEnvironment, Ty, TyBuilder, TyExt, TyKind, VariableKind,
 };
 
 impl<'a> InferenceContext<'a> {
@@ -24,32 +26,20 @@ impl<'a> InferenceContext<'a> {
     where
         T::Result: HasInterner<Interner = Interner>,
     {
-        // try to resolve obligations before canonicalizing, since this might
-        // result in new knowledge about variables
-        self.resolve_obligations_as_possible();
         self.table.canonicalize(t)
     }
 }
 
 #[derive(Debug, Clone)]
-pub(super) struct Canonicalized<T>
+pub(crate) struct Canonicalized<T>
 where
     T: HasInterner<Interner = Interner>,
 {
-    pub(super) value: Canonical<T>,
+    pub(crate) value: Canonical<T>,
     free_vars: Vec<GenericArg>,
 }
 
 impl<T: HasInterner<Interner = Interner>> Canonicalized<T> {
-    /// this method is wrong and shouldn't exist
-    pub(super) fn decanonicalize_ty(&self, table: &mut InferenceTable, ty: Canonical<Ty>) -> Ty {
-        let mut vars = self.free_vars.clone();
-        while ty.binders.len(&Interner) > vars.len() {
-            vars.push(table.new_type_var().cast(&Interner));
-        }
-        chalk_ir::Substitute::apply(&vars, ty.value, &Interner)
-    }
-
     pub(super) fn apply_solution(
         &self,
         ctx: &mut InferenceTable,
@@ -57,25 +47,25 @@ impl<T: HasInterner<Interner = Interner>> Canonicalized<T> {
     ) {
         // the solution may contain new variables, which we need to convert to new inference vars
         let new_vars = Substitution::from_iter(
-            &Interner,
-            solution.binders.iter(&Interner).map(|k| match k.kind {
-                VariableKind::Ty(TyVariableKind::General) => ctx.new_type_var().cast(&Interner),
-                VariableKind::Ty(TyVariableKind::Integer) => ctx.new_integer_var().cast(&Interner),
-                VariableKind::Ty(TyVariableKind::Float) => ctx.new_float_var().cast(&Interner),
+            Interner,
+            solution.binders.iter(Interner).map(|k| match k.kind {
+                VariableKind::Ty(TyVariableKind::General) => ctx.new_type_var().cast(Interner),
+                VariableKind::Ty(TyVariableKind::Integer) => ctx.new_integer_var().cast(Interner),
+                VariableKind::Ty(TyVariableKind::Float) => ctx.new_float_var().cast(Interner),
                 // Chalk can sometimes return new lifetime variables. We just use the static lifetime everywhere
-                VariableKind::Lifetime => static_lifetime().cast(&Interner),
+                VariableKind::Lifetime => static_lifetime().cast(Interner),
                 _ => panic!("const variable in solution"),
             }),
         );
-        for (i, v) in solution.value.iter(&Interner).enumerate() {
+        for (i, v) in solution.value.iter(Interner).enumerate() {
             let var = self.free_vars[i].clone();
-            if let Some(ty) = v.ty(&Interner) {
+            if let Some(ty) = v.ty(Interner) {
                 // eagerly replace projections in the type; we may be getting types
                 // e.g. from where clauses where this hasn't happened yet
-                let ty = ctx.normalize_associated_types_in(new_vars.apply(ty.clone(), &Interner));
-                ctx.unify(var.assert_ty_ref(&Interner), &ty);
+                let ty = ctx.normalize_associated_types_in(new_vars.apply(ty.clone(), Interner));
+                ctx.unify(var.assert_ty_ref(Interner), &ty);
             } else {
-                let _ = ctx.try_unify(&var, &new_vars.apply(v.clone(), &Interner));
+                let _ = ctx.try_unify(&var, &new_vars.apply(v.clone(), Interner));
             }
         }
     }
@@ -96,39 +86,39 @@ pub(crate) fn unify(
 ) -> Option<Substitution> {
     let mut table = InferenceTable::new(db, env);
     let vars = Substitution::from_iter(
-        &Interner,
+        Interner,
         tys.binders
-            .iter(&Interner)
+            .iter(Interner)
             // we always use type vars here because we want everything to
             // fallback to Unknown in the end (kind of hacky, as below)
             .map(|_| table.new_type_var()),
     );
-    let ty1_with_vars = vars.apply(tys.value.0.clone(), &Interner);
-    let ty2_with_vars = vars.apply(tys.value.1.clone(), &Interner);
+    let ty1_with_vars = vars.apply(tys.value.0.clone(), Interner);
+    let ty2_with_vars = vars.apply(tys.value.1.clone(), Interner);
     if !table.unify(&ty1_with_vars, &ty2_with_vars) {
         return None;
     }
     // default any type vars that weren't unified back to their original bound vars
     // (kind of hacky)
     let find_var = |iv| {
-        vars.iter(&Interner).position(|v| match v.interned() {
-            chalk_ir::GenericArgData::Ty(ty) => ty.inference_var(&Interner),
-            chalk_ir::GenericArgData::Lifetime(lt) => lt.inference_var(&Interner),
-            chalk_ir::GenericArgData::Const(c) => c.inference_var(&Interner),
+        vars.iter(Interner).position(|v| match v.interned() {
+            chalk_ir::GenericArgData::Ty(ty) => ty.inference_var(Interner),
+            chalk_ir::GenericArgData::Lifetime(lt) => lt.inference_var(Interner),
+            chalk_ir::GenericArgData::Const(c) => c.inference_var(Interner),
         } == Some(iv))
     };
     let fallback = |iv, kind, default, binder| match kind {
         chalk_ir::VariableKind::Ty(_ty_kind) => find_var(iv)
-            .map_or(default, |i| BoundVar::new(binder, i).to_ty(&Interner).cast(&Interner)),
+            .map_or(default, |i| BoundVar::new(binder, i).to_ty(Interner).cast(Interner)),
         chalk_ir::VariableKind::Lifetime => find_var(iv)
-            .map_or(default, |i| BoundVar::new(binder, i).to_lifetime(&Interner).cast(&Interner)),
+            .map_or(default, |i| BoundVar::new(binder, i).to_lifetime(Interner).cast(Interner)),
         chalk_ir::VariableKind::Const(ty) => find_var(iv)
-            .map_or(default, |i| BoundVar::new(binder, i).to_const(&Interner, ty).cast(&Interner)),
+            .map_or(default, |i| BoundVar::new(binder, i).to_const(Interner, ty).cast(Interner)),
     };
     Some(Substitution::from_iter(
-        &Interner,
-        vars.iter(&Interner)
-            .map(|v| table.resolve_with_fallback(v.assert_ty_ref(&Interner).clone(), fallback)),
+        Interner,
+        vars.iter(Interner)
+            .map(|v| table.resolve_with_fallback(v.assert_ty_ref(Interner).clone(), &fallback)),
     ))
 }
 
@@ -150,7 +140,8 @@ pub(crate) struct InferenceTable<'a> {
 
 pub(crate) struct InferenceTableSnapshot {
     var_table_snapshot: chalk_solve::infer::InferenceSnapshot<Interner>,
-    // FIXME: snapshot type_variable_table, pending_obligations?
+    // FIXME: snapshot pending_obligations?
+    type_variable_table_snapshot: Vec<TypeVariableData>,
 }
 
 impl<'a> InferenceTable<'a> {
@@ -200,21 +191,24 @@ impl<'a> InferenceTable<'a> {
             TyVariableKind::Integer => TyKind::Scalar(Scalar::Int(IntTy::I32)),
             TyVariableKind::Float => TyKind::Scalar(Scalar::Float(FloatTy::F64)),
         }
-        .intern(&Interner)
+        .intern(Interner)
     }
 
-    pub(super) fn canonicalize<T: Fold<Interner> + HasInterner<Interner = Interner>>(
+    pub(crate) fn canonicalize<T: Fold<Interner> + HasInterner<Interner = Interner>>(
         &mut self,
         t: T,
     ) -> Canonicalized<T::Result>
     where
         T::Result: HasInterner<Interner = Interner>,
     {
-        let result = self.var_unification_table.canonicalize(&Interner, t);
+        // try to resolve obligations before canonicalizing, since this might
+        // result in new knowledge about variables
+        self.resolve_obligations_as_possible();
+        let result = self.var_unification_table.canonicalize(Interner, t);
         let free_vars = result
             .free_vars
             .into_iter()
-            .map(|free_var| free_var.to_generic_arg(&Interner))
+            .map(|free_var| free_var.to_generic_arg(Interner))
             .collect();
         Canonicalized { value: result.quantified, free_vars }
     }
@@ -225,10 +219,10 @@ impl<'a> InferenceTable<'a> {
     /// type annotation (e.g. from a let type annotation, field type or function
     /// call). `make_ty` handles this already, but e.g. for field types we need
     /// to do it as well.
-    pub(super) fn normalize_associated_types_in(&mut self, ty: Ty) -> Ty {
+    pub(crate) fn normalize_associated_types_in(&mut self, ty: Ty) -> Ty {
         fold_tys(
             ty,
-            |ty, _| match ty.kind(&Interner) {
+            |ty, _| match ty.kind(Interner) {
                 TyKind::Alias(AliasTy::Projection(proj_ty)) => {
                     self.normalize_projection_ty(proj_ty.clone())
                 }
@@ -238,10 +232,10 @@ impl<'a> InferenceTable<'a> {
         )
     }
 
-    pub(super) fn normalize_projection_ty(&mut self, proj_ty: ProjectionTy) -> Ty {
+    pub(crate) fn normalize_projection_ty(&mut self, proj_ty: ProjectionTy) -> Ty {
         let var = self.new_type_var();
         let alias_eq = AliasEq { alias: AliasTy::Projection(proj_ty), ty: var.clone() };
-        let obligation = alias_eq.cast(&Interner);
+        let obligation = alias_eq.cast(Interner);
         self.register_obligation(obligation);
         var
     }
@@ -259,7 +253,7 @@ impl<'a> InferenceTable<'a> {
         self.extend_type_variable_table(var.index() as usize);
         assert_eq!(var.index() as usize, self.type_variable_table.len() - 1);
         self.type_variable_table[var.index() as usize].diverging = diverging;
-        var.to_ty_with_kind(&Interner, kind)
+        var.to_ty_with_kind(Interner, kind)
     }
 
     pub(crate) fn new_type_var(&mut self) -> Ty {
@@ -280,18 +274,18 @@ impl<'a> InferenceTable<'a> {
 
     pub(crate) fn new_const_var(&mut self, ty: Ty) -> Const {
         let var = self.var_unification_table.new_variable(UniverseIndex::ROOT);
-        var.to_const(&Interner, ty)
+        var.to_const(Interner, ty)
     }
 
     pub(crate) fn new_lifetime_var(&mut self) -> Lifetime {
         let var = self.var_unification_table.new_variable(UniverseIndex::ROOT);
-        var.to_lifetime(&Interner)
+        var.to_lifetime(Interner)
     }
 
     pub(crate) fn resolve_with_fallback<T>(
         &mut self,
         t: T,
-        fallback: impl Fn(InferenceVar, VariableKind, GenericArg, DebruijnIndex) -> GenericArg,
+        fallback: &dyn Fn(InferenceVar, VariableKind, GenericArg, DebruijnIndex) -> GenericArg,
     ) -> T::Result
     where
         T: HasInterner<Interner = Interner> + Fold<Interner>,
@@ -299,11 +293,18 @@ impl<'a> InferenceTable<'a> {
         self.resolve_with_fallback_inner(&mut Vec::new(), t, &fallback)
     }
 
+    pub(crate) fn instantiate_canonical<T>(&mut self, canonical: Canonical<T>) -> T::Result
+    where
+        T: HasInterner<Interner = Interner> + Fold<Interner> + std::fmt::Debug,
+    {
+        self.var_unification_table.instantiate_canonical(Interner, canonical)
+    }
+
     fn resolve_with_fallback_inner<T>(
         &mut self,
         var_stack: &mut Vec<InferenceVar>,
         t: T,
-        fallback: &impl Fn(InferenceVar, VariableKind, GenericArg, DebruijnIndex) -> GenericArg,
+        fallback: &dyn Fn(InferenceVar, VariableKind, GenericArg, DebruijnIndex) -> GenericArg,
     ) -> T::Result
     where
         T: HasInterner<Interner = Interner> + Fold<Interner>,
@@ -319,15 +320,14 @@ impl<'a> InferenceTable<'a> {
     where
         T: HasInterner<Interner = Interner> + Fold<Interner>,
     {
-        self.resolve_with_fallback(t, |_, _, d, _| d)
+        self.resolve_with_fallback(t, &|_, _, d, _| d)
     }
 
     /// Unify two types and register new trait goals that arise from that.
     pub(crate) fn unify(&mut self, ty1: &Ty, ty2: &Ty) -> bool {
-        let result = if let Ok(r) = self.try_unify(ty1, ty2) {
-            r
-        } else {
-            return false;
+        let result = match self.try_unify(ty1, ty2) {
+            Ok(r) => r,
+            Err(_) => return false,
         };
         self.register_infer_ok(result);
         true
@@ -337,7 +337,7 @@ impl<'a> InferenceTable<'a> {
     /// caller needs to deal with them.
     pub(crate) fn try_unify<T: Zip<Interner>>(&mut self, t1: &T, t2: &T) -> InferResult<()> {
         match self.var_unification_table.relate(
-            &Interner,
+            Interner,
             &self.db,
             &self.trait_env.env,
             chalk_ir::Variance::Invariant,
@@ -352,16 +352,29 @@ impl<'a> InferenceTable<'a> {
     /// If `ty` is a type variable with known type, returns that type;
     /// otherwise, return ty.
     pub(crate) fn resolve_ty_shallow(&mut self, ty: &Ty) -> Ty {
-        self.var_unification_table.normalize_ty_shallow(&Interner, ty).unwrap_or_else(|| ty.clone())
+        self.resolve_obligations_as_possible();
+        self.var_unification_table.normalize_ty_shallow(Interner, ty).unwrap_or_else(|| ty.clone())
     }
 
     pub(crate) fn snapshot(&mut self) -> InferenceTableSnapshot {
-        let snapshot = self.var_unification_table.snapshot();
-        InferenceTableSnapshot { var_table_snapshot: snapshot }
+        let var_table_snapshot = self.var_unification_table.snapshot();
+        let type_variable_table_snapshot = self.type_variable_table.clone();
+        InferenceTableSnapshot { var_table_snapshot, type_variable_table_snapshot }
     }
 
     pub(crate) fn rollback_to(&mut self, snapshot: InferenceTableSnapshot) {
         self.var_unification_table.rollback_to(snapshot.var_table_snapshot);
+        self.type_variable_table = snapshot.type_variable_table_snapshot;
+    }
+
+    /// Checks an obligation without registering it. Useful mostly to check
+    /// whether a trait *might* be implemented before deciding to 'lock in' the
+    /// choice (during e.g. method resolution or deref).
+    pub(crate) fn try_obligation(&mut self, goal: Goal) -> Option<Solution> {
+        let in_env = InEnvironment::new(&self.trait_env.env, goal);
+        let canonicalized = self.canonicalize(in_env);
+        let solution = self.db.trait_solve(self.trait_env.krate, canonicalized.value);
+        solution
     }
 
     pub(crate) fn register_obligation(&mut self, goal: Goal) {
@@ -396,7 +409,7 @@ impl<'a> InferenceTable<'a> {
                 let uncanonical = chalk_ir::Substitute::apply(
                     &canonicalized.free_vars,
                     canonicalized.value.value,
-                    &Interner,
+                    Interner,
                 );
                 self.register_obligation_in_env(uncanonical);
             }
@@ -412,13 +425,15 @@ impl<'a> InferenceTable<'a> {
             table: &'a mut InferenceTable<'b>,
             highest_known_var: InferenceVar,
         }
-        impl<'a, 'b> Folder<'static, Interner> for VarFudger<'a, 'b> {
-            fn as_dyn(&mut self) -> &mut dyn Folder<'static, Interner> {
+        impl<'a, 'b> Folder<Interner> for VarFudger<'a, 'b> {
+            type Error = NoSolution;
+
+            fn as_dyn(&mut self) -> &mut dyn Folder<Interner, Error = Self::Error> {
                 self
             }
 
-            fn interner(&self) -> &'static Interner {
-                &Interner
+            fn interner(&self) -> Interner {
+                Interner
             }
 
             fn fold_inference_ty(
@@ -428,7 +443,7 @@ impl<'a> InferenceTable<'a> {
                 _outer_binder: chalk_ir::DebruijnIndex,
             ) -> chalk_ir::Fallible<chalk_ir::Ty<Interner>> {
                 Ok(if var < self.highest_known_var {
-                    var.to_ty(&Interner, kind)
+                    var.to_ty(Interner, kind)
                 } else {
                     self.table.new_type_var()
                 })
@@ -440,7 +455,7 @@ impl<'a> InferenceTable<'a> {
                 _outer_binder: chalk_ir::DebruijnIndex,
             ) -> chalk_ir::Fallible<chalk_ir::Lifetime<Interner>> {
                 Ok(if var < self.highest_known_var {
-                    var.to_lifetime(&Interner)
+                    var.to_lifetime(Interner)
                 } else {
                     self.table.new_lifetime_var()
                 })
@@ -453,7 +468,7 @@ impl<'a> InferenceTable<'a> {
                 _outer_binder: chalk_ir::DebruijnIndex,
             ) -> chalk_ir::Fallible<chalk_ir::Const<Interner>> {
                 Ok(if var < self.highest_known_var {
-                    var.to_const(&Interner, ty)
+                    var.to_const(Interner, ty)
                 } else {
                     self.table.new_const_var(ty)
                 })
@@ -461,8 +476,7 @@ impl<'a> InferenceTable<'a> {
         }
 
         let snapshot = self.snapshot();
-        let highest_known_var =
-            self.new_type_var().inference_var(&Interner).expect("inference_var");
+        let highest_known_var = self.new_type_var().inference_var(Interner).expect("inference_var");
         let result = f(self);
         self.rollback_to(snapshot);
         result
@@ -476,10 +490,10 @@ impl<'a> InferenceTable<'a> {
     /// again -- it'll give the same result as last time.
     fn check_changed(&mut self, canonicalized: &Canonicalized<InEnvironment<Goal>>) -> bool {
         canonicalized.free_vars.iter().any(|var| {
-            let iv = match var.data(&Interner) {
-                chalk_ir::GenericArgData::Ty(ty) => ty.inference_var(&Interner),
-                chalk_ir::GenericArgData::Lifetime(lt) => lt.inference_var(&Interner),
-                chalk_ir::GenericArgData::Const(c) => c.inference_var(&Interner),
+            let iv = match var.data(Interner) {
+                chalk_ir::GenericArgData::Ty(ty) => ty.inference_var(Interner),
+                chalk_ir::GenericArgData::Lifetime(lt) => lt.inference_var(Interner),
+                chalk_ir::GenericArgData::Const(c) => c.inference_var(Interner),
             }
             .expect("free var is not inference var");
             if self.var_unification_table.probe_var(iv).is_some() {
@@ -522,6 +536,51 @@ impl<'a> InferenceTable<'a> {
             }
         }
     }
+
+    pub(crate) fn callable_sig(&mut self, ty: &Ty, num_args: usize) -> Option<(Vec<Ty>, Ty)> {
+        match ty.callable_sig(self.db) {
+            Some(sig) => Some((sig.params().to_vec(), sig.ret().clone())),
+            None => self.callable_sig_from_fn_trait(ty, num_args),
+        }
+    }
+
+    fn callable_sig_from_fn_trait(&mut self, ty: &Ty, num_args: usize) -> Option<(Vec<Ty>, Ty)> {
+        let krate = self.trait_env.krate;
+        let fn_once_trait = FnTrait::FnOnce.get_id(self.db, krate)?;
+        let output_assoc_type =
+            self.db.trait_data(fn_once_trait).associated_type_by_name(&name![Output])?;
+
+        let mut arg_tys = vec![];
+        let arg_ty = TyBuilder::tuple(num_args)
+            .fill(iter::repeat_with(|| {
+                let arg = self.new_type_var();
+                arg_tys.push(arg.clone());
+                arg
+            }))
+            .build();
+
+        let projection = {
+            let b = TyBuilder::assoc_type_projection(self.db, output_assoc_type);
+            if b.remaining() != 2 {
+                return None;
+            }
+            b.push(ty.clone()).push(arg_ty).build()
+        };
+
+        let trait_env = self.trait_env.env.clone();
+        let obligation = InEnvironment {
+            goal: projection.trait_ref(self.db).cast(Interner),
+            environment: trait_env,
+        };
+        let canonical = self.canonicalize(obligation.clone());
+        if self.db.trait_solve(krate, canonical.value.cast(Interner)).is_some() {
+            self.register_obligation(obligation.goal);
+            let return_ty = self.normalize_projection_ty(projection);
+            Some((arg_tys, return_ty))
+        } else {
+            None
+        }
+    }
 }
 
 impl<'a> fmt::Debug for InferenceTable<'a> {
@@ -539,7 +598,7 @@ mod resolve {
     use chalk_ir::{
         cast::Cast,
         fold::{Fold, Folder},
-        Fallible,
+        Fallible, NoSolution,
     };
     use hir_def::type_ref::ConstScalar;
 
@@ -548,16 +607,18 @@ mod resolve {
         pub(super) var_stack: &'a mut Vec<InferenceVar>,
         pub(super) fallback: F,
     }
-    impl<'a, 'b, 'i, F> Folder<'i, Interner> for Resolver<'a, 'b, F>
+    impl<'a, 'b, 'i, F> Folder<Interner> for Resolver<'a, 'b, F>
     where
         F: Fn(InferenceVar, VariableKind, GenericArg, DebruijnIndex) -> GenericArg + 'i,
     {
-        fn as_dyn(&mut self) -> &mut dyn Folder<'i, Interner> {
+        type Error = NoSolution;
+
+        fn as_dyn(&mut self) -> &mut dyn Folder<Interner, Error = Self::Error> {
             self
         }
 
-        fn interner(&self) -> &'i Interner {
-            &Interner
+        fn interner(&self) -> Interner {
+            Interner
         }
 
         fn fold_inference_ty(
@@ -569,9 +630,9 @@ mod resolve {
             let var = self.table.var_unification_table.inference_var_root(var);
             if self.var_stack.contains(&var) {
                 // recursive type
-                let default = self.table.fallback_value(var, kind).cast(&Interner);
+                let default = self.table.fallback_value(var, kind).cast(Interner);
                 return Ok((self.fallback)(var, VariableKind::Ty(kind), default, outer_binder)
-                    .assert_ty_ref(&Interner)
+                    .assert_ty_ref(Interner)
                     .clone());
             }
             let result = if let Some(known_ty) = self.table.var_unification_table.probe_var(var) {
@@ -580,11 +641,11 @@ mod resolve {
                 let result =
                     known_ty.fold_with(self, outer_binder).expect("fold failed unexpectedly");
                 self.var_stack.pop();
-                result.assert_ty_ref(&Interner).clone()
+                result.assert_ty_ref(Interner).clone()
             } else {
-                let default = self.table.fallback_value(var, kind).cast(&Interner);
+                let default = self.table.fallback_value(var, kind).cast(Interner);
                 (self.fallback)(var, VariableKind::Ty(kind), default, outer_binder)
-                    .assert_ty_ref(&Interner)
+                    .assert_ty_ref(Interner)
                     .clone()
             };
             Ok(result)
@@ -601,12 +662,12 @@ mod resolve {
                 ty: ty.clone(),
                 value: ConstValue::Concrete(ConcreteConst { interned: ConstScalar::Unknown }),
             }
-            .intern(&Interner)
-            .cast(&Interner);
+            .intern(Interner)
+            .cast(Interner);
             if self.var_stack.contains(&var) {
                 // recursive
                 return Ok((self.fallback)(var, VariableKind::Const(ty), default, outer_binder)
-                    .assert_const_ref(&Interner)
+                    .assert_const_ref(Interner)
                     .clone());
             }
             let result = if let Some(known_ty) = self.table.var_unification_table.probe_var(var) {
@@ -615,10 +676,10 @@ mod resolve {
                 let result =
                     known_ty.fold_with(self, outer_binder).expect("fold failed unexpectedly");
                 self.var_stack.pop();
-                result.assert_const_ref(&Interner).clone()
+                result.assert_const_ref(Interner).clone()
             } else {
                 (self.fallback)(var, VariableKind::Const(ty), default, outer_binder)
-                    .assert_const_ref(&Interner)
+                    .assert_const_ref(Interner)
                     .clone()
             };
             Ok(result)
